@@ -5,6 +5,7 @@
 #include <cstdio>
 
 #include <OpenImageIO/filesystem.h>
+#include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imageio.h>
 
 #include "bmp_pvt.h"
@@ -261,7 +262,7 @@ BmpInput::open(const std::string& name, ImageSpec& newspec,
     // Default presumption is that a BMP file is meant to look reasonable on a
     // display, so assume it's sRGB. This is not really correct -- see the
     // comments below.
-    m_spec.attribute("oiio:ColorSpace", "sRGB");
+    m_spec.attribute("oiio:ColorSpace", "srgb_rec709_scene");
 #if 0
     if (m_dib_header.size >= WINDOWS_V4
         && m_dib_header.cs_type == CSType::CalibratedRGB) {
@@ -313,7 +314,7 @@ BmpInput::read_rle_image()
         }
         if (y >= m_spec.height) {  // out of y bounds
             errorfmt(
-                "BMP might be corrupted, it is referencing an out-of-bounds pixel coordinte ({},{})",
+                "BMP might be corrupted, it is referencing an out-of-bounds pixel coordinate ({},{})",
                 x, y);
             ok = false;
             break;
